library angular2.src.core.application;
import 'package:angular2/di.dart' show Injector, bind, OpaqueToken;
import 'package:angular2/src/facade/lang.dart' show Type, FIELD, isBlank, isPresent, BaseException, assertionsEnabled, print;
import 'package:angular2/src/facade/dom.dart' show DOM, Element;
import './compiler/compiler.dart' show Compiler, CompilerCache;
import './compiler/view.dart' show ProtoView;
import 'package:angular2/src/reflection/reflection.dart' show Reflector, reflector;
import 'package:angular2/change_detection.dart' show Parser, Lexer, ChangeDetection, dynamicChangeDetection, jitChangeDetection;
import './compiler/template_loader.dart' show TemplateLoader;
import './compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import './compiler/directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/core/zone/vm_turn_zone.dart' show VmTurnZone;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show ShadowDomStrategy, NativeShadowDomStrategy;
import 'package:angular2/src/core/compiler/xhr/xhr.dart' show XHR;
import 'package:angular2/src/core/compiler/xhr/xhr_impl.dart' show XHRImpl;
Injector _rootInjector;
var _rootBindings = [bind(Reflector).toValue(reflector), bind(ChangeDetection).toValue(dynamicChangeDetection), Compiler, CompilerCache, TemplateLoader, DirectiveMetadataReader, Parser, Lexer, bind(ShadowDomStrategy).toValue(new NativeShadowDomStrategy()), bind(XHR).toValue(new XHRImpl())];
var appViewToken = new OpaqueToken('AppView');
var appChangeDetectorToken = new OpaqueToken('AppChangeDetector');
var appElementToken = new OpaqueToken('AppElement');
var appComponentAnnotatedTypeToken = new OpaqueToken('AppComponentAnnotatedType');
var appDocumentToken = new OpaqueToken('AppDocument');
_injectorBindings(appComponentType) {
  return [bind(appDocumentToken).toValue(DOM.defaultDoc()), bind(appComponentAnnotatedTypeToken).toFactory((reader) {
    return reader.read(appComponentType);
  }, [DirectiveMetadataReader]), bind(appElementToken).toFactory((appComponentAnnotatedType, appDocument) {
    var selector = appComponentAnnotatedType.annotation.selector;
    var element = DOM.querySelector(appDocument, selector);
    if (isBlank(element)) {
      throw new BaseException('''The app selector \"${selector}\" did not match any elements''');
    }
    return element;
  }, [appComponentAnnotatedTypeToken, appDocumentToken]), bind(appViewToken).toAsyncFactory((changeDetection, compiler, injector, appElement, appComponentAnnotatedType, strategy) {
    return compiler.compile(appComponentAnnotatedType.type, null).then((protoView) {
      var appProtoView = ProtoView.createRootProtoView(protoView, appElement, appComponentAnnotatedType, changeDetection.createProtoChangeDetector('root'), strategy);
      var view = appProtoView.instantiate(null);
      view.hydrate(injector, null, new Object());
      return view;
    });
  }, [ChangeDetection, Compiler, Injector, appElementToken, appComponentAnnotatedTypeToken, ShadowDomStrategy]), bind(appChangeDetectorToken).toFactory((rootView) {
    return rootView.changeDetector;
  }, [appViewToken]), bind(appComponentType).toFactory((rootView) {
    return rootView.elementInjectors[0].getComponent();
  }, [appViewToken]), bind(LifeCycle).toFactory((cd) {
    return new LifeCycle(cd, assertionsEnabled());
  }, [appChangeDetectorToken])];
}
_createVmZone(Function givenReporter) {
  var defaultErrorReporter = (exception, stackTrace) {
    var longStackTrace = ListWrapper.join(stackTrace, "\n\n-----async gap-----\n");
    print('''${exception}\n\n${longStackTrace}''');
    throw exception;
  };
  var reporter = isPresent(givenReporter) ? givenReporter : defaultErrorReporter;
  var zone = new VmTurnZone(enableLongStackTrace: assertionsEnabled());
  zone.initCallbacks(onErrorHandler: reporter);
  return zone;
}
bootstrap(Type appComponentType, [bindings = null, givenBootstrapErrorReporter = null]) {
  var bootstrapProcess = PromiseWrapper.completer();
  var zone = _createVmZone(givenBootstrapErrorReporter);
  zone.run(() {
    var appInjector = _createAppInjector(appComponentType, bindings);
    PromiseWrapper.then(appInjector.asyncGet(LifeCycle), (lc) {
      lc.registerWith(zone);
      lc.tick();
      bootstrapProcess.complete(appInjector);
    }, (err) {
      bootstrapProcess.reject(err);
    });
  });
  return bootstrapProcess.promise;
}
Injector _createAppInjector(Type appComponentType, List bindings) {
  if (isBlank(_rootInjector))
    _rootInjector = new Injector(_rootBindings);
  var mergedBindings = isPresent(bindings) ? ListWrapper.concat(_injectorBindings(appComponentType), bindings) : _injectorBindings(appComponentType);
  return _rootInjector.createChild(mergedBindings);
}
