library angular2.src.core.compiler.pipeline.compile_element;
import 'package:angular2/src/facade/collection.dart' show List, Map, ListWrapper, MapWrapper;
import 'package:angular2/src/facade/dom.dart' show Element, DOM;
import 'package:angular2/src/facade/lang.dart' show int, isBlank, isPresent, Type;
import '../directive_metadata.dart' show DirectiveMetadata;
import '../../annotations/annotations.dart' show Decorator;
import '../../annotations/annotations.dart' show Component;
import '../../annotations/annotations.dart' show Template;
import '../element_binder.dart' show ElementBinder;
import '../element_injector.dart' show ProtoElementInjector;
import '../view.dart' show ProtoView;
import 'package:angular2/change_detection.dart' show AST;
class CompileElement {
Element element;
Map _attrs;
List _classList;
Map textNodeBindings;
Map propertyBindings;
Map eventBindings;
Map variableBindings;
List decoratorDirectives;
DirectiveMetadata templateDirective;
DirectiveMetadata componentDirective;
List _allDirectives;
bool isViewRoot;
bool hasBindings;
ProtoView inheritedProtoView;
ProtoElementInjector inheritedProtoElementInjector;
ElementBinder inheritedElementBinder;
num distanceToParentInjector;
bool compileChildren;
CompileElement(Element element) {
  this.element = element;
  this._attrs = null;
  this._classList = null;
  this.textNodeBindings = null;
  this.propertyBindings = null;
  this.eventBindings = null;
  this.variableBindings = null;
  this.decoratorDirectives = null;
  this.templateDirective = null;
  this.componentDirective = null;
  this._allDirectives = null;
  this.isViewRoot = false;
  this.hasBindings = false;
  this.inheritedProtoView = null;
  this.inheritedProtoElementInjector = null;
  this.inheritedElementBinder = null;
  this.distanceToParentInjector = 0;
  this.compileChildren = true;
}
refreshAttrs() {
  this._attrs = null;
}
Map attrs() {
  if (isBlank(this._attrs)) {
    this._attrs = DOM.attributeMap(this.element);
  }
  return this._attrs;
}
refreshClassList() {
  this._classList = null;
}
List classList() {
  if (isBlank(this._classList)) {
    this._classList = ListWrapper.create();
    var elClassList = DOM.classList(this.element);
    for (var i = 0; i < elClassList.length; i++) {
      ListWrapper.push(this._classList, elClassList[i]);
    }
  }
  return this._classList;
}
addTextNodeBinding(int indexInParent, AST expression) {
  if (isBlank(this.textNodeBindings)) {
    this.textNodeBindings = MapWrapper.create();
  }
  MapWrapper.set(this.textNodeBindings, indexInParent, expression);
}
addPropertyBinding(String property, AST expression) {
  if (isBlank(this.propertyBindings)) {
    this.propertyBindings = MapWrapper.create();
  }
  MapWrapper.set(this.propertyBindings, property, expression);
}
addVariableBinding(String directiveName, String templateName) {
  if (isBlank(this.variableBindings)) {
    this.variableBindings = MapWrapper.create();
  }
  MapWrapper.set(this.variableBindings, templateName, directiveName);
}
addEventBinding(String eventName, AST expression) {
  if (isBlank(this.eventBindings)) {
    this.eventBindings = MapWrapper.create();
  }
  MapWrapper.set(this.eventBindings, eventName, expression);
}
addDirective(DirectiveMetadata directive) {
  var annotation = directive.annotation;
  this._allDirectives = null;
  if (annotation is Decorator) {
    if (isBlank(this.decoratorDirectives)) {
      this.decoratorDirectives = ListWrapper.create();
    }
    ListWrapper.push(this.decoratorDirectives, directive);
    if (!annotation.compileChildren) {
      this.compileChildren = false;
    }
  } else if (annotation is Template) {
    this.templateDirective = directive;
  } else if (annotation is Component) {
    this.componentDirective = directive;
  }
}
List getAllDirectives() {
  if (identical(this._allDirectives, null)) {
    var directives = ListWrapper.create();
    if (isPresent(this.componentDirective)) {
      ListWrapper.push(directives, this.componentDirective);
    }
    if (isPresent(this.templateDirective)) {
      ListWrapper.push(directives, this.templateDirective);
    }
    if (isPresent(this.decoratorDirectives)) {
      directives = ListWrapper.concat(directives, this.decoratorDirectives);
    }
    this._allDirectives = directives;
  }
  return this._allDirectives;
}
}
