library angular2.src.core.compiler.pipeline.element_binder_builder;
import 'package:angular2/src/facade/lang.dart' show int, isPresent, isBlank, Type, BaseException, StringWrapper, stringify;
import 'package:angular2/src/facade/dom.dart' show Element, DOM;
import 'package:angular2/src/facade/collection.dart' show ListWrapper, List, MapWrapper, StringMapWrapper;
import 'package:angular2/src/reflection/reflection.dart' show reflector;
import 'package:angular2/change_detection.dart' show Parser, ProtoChangeDetector;
import '../../annotations/annotations.dart' show Component, Directive;
import '../directive_metadata.dart' show DirectiveMetadata;
import '../view.dart' show ProtoView, ElementPropertyMemento, DirectivePropertyMemento;
import '../element_injector.dart' show ProtoElementInjector;
import '../element_binder.dart' show ElementBinder;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
const CLASS_PREFIX = 'class.';
var classSettersCache = StringMapWrapper.create();
classSetterFactory(String className) {
  var setterFn = StringMapWrapper.get(classSettersCache, className);
  if (isBlank(setterFn)) {
    setterFn = (Element element, value) {
      if (value) {
        DOM.addClass(element, className);
      } else {
        DOM.removeClass(element, className);
      }
    };
    StringMapWrapper.set(classSettersCache, className, setterFn);
  }
  return setterFn;
}
class ElementBinderBuilder extends CompileStep {
process(CompileElement parent, CompileElement current, CompileControl control) {
  var elementBinder = null;
  if (current.hasBindings) {
    var protoView = current.inheritedProtoView;
    elementBinder = protoView.bindElement(current.inheritedProtoElementInjector, current.componentDirective, current.templateDirective);
    if (isPresent(current.textNodeBindings)) {
      this._bindTextNodes(protoView, current);
    }
    if (isPresent(current.propertyBindings)) {
      this._bindElementProperties(protoView, current);
    }
    if (isPresent(current.eventBindings)) {
      this._bindEvents(protoView, current);
    }
    this._bindDirectiveProperties(current.getAllDirectives(), current);
  } else if (isPresent(parent)) {
    elementBinder = parent.inheritedElementBinder;
  }
  current.inheritedElementBinder = elementBinder;
}
_bindTextNodes(protoView, compileElement) {
  MapWrapper.forEach(compileElement.textNodeBindings, (expression, indexInParent) {
    protoView.bindTextNode(indexInParent, expression);
  });
}
_bindElementProperties(protoView, compileElement) {
  MapWrapper.forEach(compileElement.propertyBindings, (expression, property) {
    var setterFn;
    if (StringWrapper.startsWith(property, CLASS_PREFIX)) {
      setterFn = classSetterFactory(StringWrapper.substring(property, CLASS_PREFIX.length));
    } else if (DOM.hasProperty(compileElement.element, property)) {
      setterFn = reflector.setter(property);
    }
    if (isPresent(setterFn)) {
      protoView.bindElementProperty(expression.ast, property, setterFn);
    }
  });
}
_bindEvents(protoView, compileElement) {
  MapWrapper.forEach(compileElement.eventBindings, (expression, eventName) {
    protoView.bindEvent(eventName, expression);
  });
}
_bindDirectiveProperties(List directives, CompileElement compileElement) {
  var protoView = compileElement.inheritedProtoView;
  for (var directiveIndex = 0; directiveIndex < directives.length; directiveIndex++) {
    var directive = ListWrapper.get(directives, directiveIndex);
    var annotation = directive.annotation;
    if (isBlank(annotation.bind))
      continue;
    StringMapWrapper.forEach(annotation.bind, (dirProp, elProp) {
      var expression = isPresent(compileElement.propertyBindings) ? MapWrapper.get(compileElement.propertyBindings, elProp) : null;
      if (isBlank(expression)) {
        throw new BaseException("No element binding found for property '" + elProp + "' which is required by directive '" + stringify(directive.type) + "'");
      }
      var len = dirProp.length;
      var dirBindingName = dirProp;
      var isContentWatch = identical(dirProp[len - 2], '[') && identical(dirProp[len - 1], ']');
      if (isContentWatch)
        dirBindingName = dirProp.substring(0, len - 2);
      protoView.bindDirectiveProperty(directiveIndex, expression, dirBindingName, reflector.setter(dirBindingName), isContentWatch);
    });
  }
}
}
