library angular2.src.core.compiler.pipeline.property_binding_parser;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank, RegExpWrapper, BaseException;
import 'package:angular2/src/facade/collection.dart' show MapWrapper;
import 'package:angular2/src/facade/dom.dart' show TemplateElement;
import 'package:angular2/change_detection.dart' show Parser, AST, ExpressionWithSource;
import './compile_step.dart' show CompileStep;
import './compile_element.dart' show CompileElement;
import './compile_control.dart' show CompileControl;
var BIND_NAME_REGEXP = RegExpWrapper.create('^(?:(?:(bind)|(var)|(on))-(.+))|\\[([^\\]]+)\\]|\\(([^\\)]+)\\)');
class PropertyBindingParser extends CompileStep {
Parser _parser;
dynamic _compilationUnit;
PropertyBindingParser(Parser parser, dynamic compilationUnit) {
  this._parser = parser;
  this._compilationUnit = compilationUnit;
}
process(CompileElement parent, CompileElement current, CompileControl control) {
  var attrs = current.attrs();
  MapWrapper.forEach(attrs, (attrValue, attrName) {
    var bindParts = RegExpWrapper.firstMatch(BIND_NAME_REGEXP, attrName);
    if (isPresent(bindParts)) {
      if (isPresent(bindParts[1])) {
        current.addPropertyBinding(bindParts[4], this._parseBinding(attrValue));
      } else if (isPresent(bindParts[2])) {
        if (!(current.element is TemplateElement)) {
          throw new BaseException('var-* is only allowed on <template> elements!');
        }
        current.addVariableBinding(bindParts[4], attrValue);
      } else if (isPresent(bindParts[3])) {
        current.addEventBinding(bindParts[4], this._parseAction(attrValue));
      } else if (isPresent(bindParts[5])) {
        current.addPropertyBinding(bindParts[5], this._parseBinding(attrValue));
      } else if (isPresent(bindParts[6])) {
        current.addEventBinding(bindParts[6], this._parseBinding(attrValue));
      }
    } else {
      var ast = this._parseInterpolation(attrValue);
      if (isPresent(ast)) {
        current.addPropertyBinding(attrName, ast);
      }
    }
  });
}
AST _parseInterpolation(String input) {
  return this._parser.parseInterpolation(input, this._compilationUnit);
}
AST _parseBinding(String input) {
  return this._parser.parseBinding(input, this._compilationUnit);
}
AST _parseAction(String input) {
  return this._parser.parseAction(input, this._compilationUnit);
}
}
