library angular2.src.core.compiler.shadow_dom_strategy;
import 'package:angular2/src/facade/lang.dart' show Type, isBlank, isPresent;
import 'package:angular2/src/facade/dom.dart' show DOM, Element, StyleElement;
import 'package:angular2/src/facade/collection.dart' show List, ListWrapper;
import './view.dart' show View;
import './shadow_dom_emulation/content_tag.dart' show Content;
import './shadow_dom_emulation/light_dom.dart' show LightDom;
import './directive_metadata.dart' show DirectiveMetadata;
import './shadow_dom_emulation/shim_css.dart' show shimCssText;
class ShadowDomStrategy {
attachTemplate(Element el, View view) {}
constructLightDom(View lightDomView, View shadowDomView, Element el) {}
List polyfillDirectives() {
  return null;
}
processTemplate(Element template, DirectiveMetadata cmpMetadata) {
  return null;
}
}
class EmulatedShadowDomStrategy extends ShadowDomStrategy {
Element _styleHost;
EmulatedShadowDomStrategy([Element styleHost = null]) {
  if (isBlank(styleHost)) {
    styleHost = DOM.defaultDoc().head;
  }
  this._styleHost = styleHost;
}
attachTemplate(Element el, View view) {
  DOM.clearNodes(el);
  moveViewNodesIntoParent(el, view);
}
constructLightDom(View lightDomView, View shadowDomView, Element el) {
  return new LightDom(lightDomView, shadowDomView, el);
}
List polyfillDirectives() {
  return [Content];
}
processTemplate(Element template, DirectiveMetadata cmpMetadata) {
  var templateRoot = DOM.templateAwareRoot(template);
  var attrName = cmpMetadata.annotation.selector;
  var styles = _detachStyles(templateRoot);
  for (var i = 0; i < styles.length; i++) {
    var style = styles[i];
    var processedCss = shimCssText(DOM.getText(style), attrName);
    DOM.setText(style, processedCss);
  }
  _attachStyles(this._styleHost, styles);
  _addAttributeToChildren(templateRoot, attrName);
}
}
class NativeShadowDomStrategy extends ShadowDomStrategy {
attachTemplate(Element el, View view) {
  moveViewNodesIntoParent(el.createShadowRoot(), view);
}
constructLightDom(View lightDomView, View shadowDomView, Element el) {
  return null;
}
List polyfillDirectives() {
  return [];
}
processTemplate(Element template, DirectiveMetadata cmpMetadata) {
  return template;
}
}
moveViewNodesIntoParent(parent, view) {
  for (var i = 0; i < view.nodes.length; ++i) {
    DOM.appendChild(parent, view.nodes[i]);
  }
}
List _detachStyles(el) {
  var nodeList = DOM.querySelectorAll(el, 'style');
  var styles = [];
  for (var i = 0; i < nodeList.length; i++) {
    var style = DOM.remove(nodeList[i]);
    ListWrapper.push(styles, style);
  }
  return styles;
}
_attachStyles(Element el, List styles) {
  var firstChild = DOM.firstChild(el);
  for (var i = styles.length - 1; i >= 0; i--) {
    var style = styles[i];
    if (isPresent(firstChild)) {
      DOM.insertBefore(firstChild, style);
    } else {
      DOM.appendChild(el, style);
    }
    firstChild = style;
  }
}
_addAttributeToChildren(el, String attrName) {
  var children = DOM.querySelectorAll(el, "*");
  for (var i = 0; i < children.length; i++) {
    var child = children[i];
    DOM.setAttribute(child, attrName, '');
  }
}
