library angular2.src.core.compiler.template_loader;
import 'package:angular2/src/facade/async.dart' show Future, PromiseWrapper;
import 'package:angular2/src/facade/lang.dart' show isBlank, isPresent, BaseException, stringify;
import 'package:angular2/src/facade/dom.dart' show TemplateElement, DOM;
import 'package:angular2/src/facade/collection.dart' show StringMapWrapper;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/compiler/directive_metadata.dart' show DirectiveMetadata;
import './xhr/xhr.dart' show XHR;
class TemplateLoader {
XHR _xhr;
var _cache;
TemplateLoader(XHR xhr) {
  this._xhr = xhr;
  this._cache = StringMapWrapper.create();
}
Future load(DirectiveMetadata cmpMetadata) {
  Component annotation = cmpMetadata.annotation;
  TemplateConfig tplConfig = annotation.template;
  if (isPresent(tplConfig.inline)) {
    var template = DOM.createTemplate(tplConfig.inline);
    return PromiseWrapper.resolve(template);
  }
  if (isPresent(tplConfig.url)) {
    var url = tplConfig.url;
    var promise = StringMapWrapper.get(this._cache, url);
    if (isBlank(promise)) {
      promise = this._xhr.get(url).then((html) {
        var template = DOM.createTemplate(html);
        return template;
      });
      StringMapWrapper.set(this._cache, url, promise);
    }
    return promise;
  }
  throw new BaseException('''No template configured for component ${stringify(cmpMetadata.type)}''');
}
}
