library angular2.src.core.compiler.view;
import 'package:angular2/src/facade/dom.dart' show DOM, Element, Node, Text, DocumentFragment, TemplateElement;
import 'package:angular2/src/facade/collection.dart' show ListWrapper, MapWrapper, StringMapWrapper, List;
import 'package:angular2/change_detection.dart' show AST, ContextWithVariableBindings, ChangeDispatcher, ProtoChangeDetector, ChangeDetector, ChangeRecord;
import './element_injector.dart' show ProtoElementInjector, ElementInjector, PreBuiltObjects;
import './binding_propagation_config.dart' show BindingPropagationConfig;
import './element_binder.dart' show ElementBinder;
import './directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/reflection/types.dart' show SetterFn;
import 'package:angular2/src/facade/lang.dart' show FIELD, IMPLEMENTS, int, isPresent, isBlank, BaseException;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import './viewport.dart' show ViewPort;
import './interfaces.dart' show OnChange;
import './shadow_dom_emulation/content_tag.dart' show Content;
import './shadow_dom_emulation/light_dom.dart' show LightDom, DestinationLightDom;
import './shadow_dom_strategy.dart' show ShadowDomStrategy;
const NG_BINDING_CLASS = 'ng-binding';
const NG_BINDING_CLASS_SELECTOR = '.ng-binding';
var NO_FORMATTERS = MapWrapper.create();
@IMPLEMENTS(ChangeDispatcher) 
class View {
List rootElementInjectors;
List elementInjectors;
List bindElements;
List textNodes;
ChangeDetector changeDetector;
List nodes;
List componentChildViews;
List viewPorts;
List preBuiltObjects;
ProtoView proto;
dynamic context;
ContextWithVariableBindings contextWithLocals;
View(ProtoView proto, List nodes, ProtoChangeDetector protoChangeDetector, Map protoContextLocals) {
  this.proto = proto;
  this.nodes = nodes;
  this.changeDetector = protoChangeDetector.instantiate(this, NO_FORMATTERS);
  this.elementInjectors = null;
  this.rootElementInjectors = null;
  this.textNodes = null;
  this.bindElements = null;
  this.componentChildViews = null;
  this.viewPorts = null;
  this.preBuiltObjects = null;
  this.context = null;
  this.contextWithLocals = (MapWrapper.size(protoContextLocals) > 0) ? new ContextWithVariableBindings(null, MapWrapper.clone(protoContextLocals)) : null;
}
init(List elementInjectors, List rootElementInjectors, List textNodes, List bindElements, List viewPorts, List preBuiltObjects, List componentChildViews) {
  this.elementInjectors = elementInjectors;
  this.rootElementInjectors = rootElementInjectors;
  this.textNodes = textNodes;
  this.bindElements = bindElements;
  this.viewPorts = viewPorts;
  this.preBuiltObjects = preBuiltObjects;
  this.componentChildViews = componentChildViews;
}
setLocal(String contextName, value) {
  if (!this.hydrated())
    throw new BaseException('Cannot set locals on dehydrated view.');
  if (!MapWrapper.contains(this.proto.variableBindings, contextName)) {
    return ;
  }
  var templateName = MapWrapper.get(this.proto.variableBindings, contextName);
  this.context.set(templateName, value);
}
hydrated() {
  return isPresent(this.context);
}
_hydrateContext(newContext) {
  if (isPresent(this.contextWithLocals)) {
    this.contextWithLocals.parent = newContext;
    this.context = this.contextWithLocals;
  } else {
    this.context = newContext;
  }
  this.changeDetector.setContext(this.context);
}
_dehydrateContext() {
  if (isPresent(this.contextWithLocals)) {
    this.contextWithLocals.clearValues();
  }
  this.context = null;
}
hydrate(Injector appInjector, ElementInjector hostElementInjector, Object context) {
  if (this.hydrated())
    throw new BaseException('The view is already hydrated.');
  this._hydrateContext(context);
  for (var i = 0; i < this.viewPorts.length; i++) {
    this.viewPorts[i].hydrate(appInjector, hostElementInjector);
  }
  var binders = this.proto.elementBinders;
  var componentChildViewIndex = 0;
  for (var i = 0; i < binders.length; ++i) {
    var componentDirective = binders[i].componentDirective;
    var shadowDomAppInjector = null;
    if (isPresent(componentDirective)) {
      var services = componentDirective.annotation.componentServices;
      if (isPresent(services))
        shadowDomAppInjector = appInjector.createChild(services);
      else {
        shadowDomAppInjector = appInjector;
      }
    } else {
      shadowDomAppInjector = null;
    }
    var elementInjector = this.elementInjectors[i];
    if (isPresent(elementInjector)) {
      elementInjector.instantiateDirectives(appInjector, shadowDomAppInjector, this.preBuiltObjects[i]);
    }
    if (isPresent(componentDirective)) {
      this.componentChildViews[componentChildViewIndex++].hydrate(shadowDomAppInjector, elementInjector, elementInjector.getComponent());
    }
  }
  for (var i = 0; i < binders.length; ++i) {
    var componentDirective = binders[i].componentDirective;
    if (isPresent(componentDirective)) {
      var lightDom = this.preBuiltObjects[i].lightDom;
      if (isPresent(lightDom)) {
        lightDom.redistribute();
      }
    }
  }
}
dehydrate() {
  for (var i = 0; i < this.componentChildViews.length; i++) {
    this.componentChildViews[i].dehydrate();
  }
  for (var i = 0; i < this.elementInjectors.length; i++) {
    if (isPresent(this.elementInjectors[i])) {
      this.elementInjectors[i].clearDirectives();
    }
  }
  if (isPresent(this.viewPorts)) {
    for (var i = 0; i < this.viewPorts.length; i++) {
      this.viewPorts[i].dehydrate();
    }
  }
  this._dehydrateContext();
}
onRecordChange(groupMemento, List records) {
  this._invokeMementos(records);
  if (groupMemento is DirectivePropertyGroupMemento) {
    this._notifyDirectiveAboutChanges(groupMemento, records);
  }
}
_invokeMementos(List records) {
  for (var i = 0; i < records.length; ++i) {
    this._invokeMementoFor(records[i]);
  }
}
_notifyDirectiveAboutChanges(groupMemento, List records) {
  var dir = groupMemento.directive(this.elementInjectors);
  if (dir is OnChange) {
    dir.onChange(this._collectChanges(records));
  }
}
_invokeMementoFor(ChangeRecord record) {
  var memento = record.bindingMemento;
  if (memento is DirectivePropertyMemento) {
    DirectivePropertyMemento directiveMemento = memento;
    directiveMemento.invoke(record, this.elementInjectors);
  } else if (memento is ElementPropertyMemento) {
    ElementPropertyMemento elementMemento = memento;
    elementMemento.invoke(record, this.bindElements);
  } else {
    num textNodeIndex = memento;
    DOM.setText(this.textNodes[textNodeIndex], record.currentValue);
  }
}
_collectChanges(List records) {
  var changes = StringMapWrapper.create();
  for (var i = 0; i < records.length; ++i) {
    var record = records[i];
    var propertyUpdate = new PropertyUpdate(record.currentValue, record.previousValue);
    StringMapWrapper.set(changes, record.bindingMemento._setterName, propertyUpdate);
  }
  return changes;
}
}
class ProtoView {
Element element;
List elementBinders;
ProtoChangeDetector protoChangeDetector;
Map variableBindings;
Map protoContextLocals;
int textNodesWithBindingCount;
int elementsWithBindingCount;
bool instantiateInPlace;
int rootBindingOffset;
bool isTemplateElement;
ShadowDomStrategy shadowDomStrategy;
ProtoView(Element template, ProtoChangeDetector protoChangeDetector, ShadowDomStrategy shadowDomStrategy) {
  this.element = template;
  this.elementBinders = [];
  this.variableBindings = MapWrapper.create();
  this.protoContextLocals = MapWrapper.create();
  this.protoChangeDetector = protoChangeDetector;
  this.textNodesWithBindingCount = 0;
  this.elementsWithBindingCount = 0;
  this.instantiateInPlace = false;
  this.rootBindingOffset = (isPresent(this.element) && DOM.hasClass(this.element, NG_BINDING_CLASS)) ? 1 : 0;
  this.isTemplateElement = this.element is TemplateElement;
  this.shadowDomStrategy = shadowDomStrategy;
}
View instantiate(ElementInjector hostElementInjector) {
  var rootElementClone = this.instantiateInPlace ? this.element : DOM.clone(this.element);
  var elementsWithBindingsDynamic;
  if (this.isTemplateElement) {
    elementsWithBindingsDynamic = DOM.querySelectorAll(rootElementClone.content, NG_BINDING_CLASS_SELECTOR);
  } else {
    elementsWithBindingsDynamic = DOM.getElementsByClassName(rootElementClone, NG_BINDING_CLASS);
  }
  var elementsWithBindings = ListWrapper.createFixedSize(elementsWithBindingsDynamic.length);
  for (var i = 0; i < elementsWithBindingsDynamic.length; ++i) {
    elementsWithBindings[i] = elementsWithBindingsDynamic[i];
  }
  var viewNodes;
  if (this.isTemplateElement) {
    var childNode = DOM.firstChild(rootElementClone.content);
    viewNodes = [];
    while (childNode != null) {
      ListWrapper.push(viewNodes, childNode);
      childNode = DOM.nextSibling(childNode);
    }
  } else {
    viewNodes = [rootElementClone];
  }
  var view = new View(this, viewNodes, this.protoChangeDetector, this.protoContextLocals);
  var binders = this.elementBinders;
  var elementInjectors = ListWrapper.createFixedSize(binders.length);
  var rootElementInjectors = [];
  var textNodes = [];
  var elementsWithPropertyBindings = [];
  var preBuiltObjects = ListWrapper.createFixedSize(binders.length);
  var viewPorts = [];
  var componentChildViews = [];
  for (var i = 0; i < binders.length; i++) {
    var binder = binders[i];
    var element;
    if (identical(i, 0) && identical(this.rootBindingOffset, 1)) {
      element = rootElementClone;
    } else {
      element = elementsWithBindings[i - this.rootBindingOffset];
    }
    var elementInjector = null;
    var protoElementInjector = binder.protoElementInjector;
    if (isPresent(protoElementInjector)) {
      if (isPresent(protoElementInjector.parent)) {
        var parentElementInjector = elementInjectors[protoElementInjector.parent.index];
        elementInjector = protoElementInjector.instantiate(parentElementInjector, null, binder.events);
      } else {
        elementInjector = protoElementInjector.instantiate(null, hostElementInjector, binder.events);
        ListWrapper.push(rootElementInjectors, elementInjector);
      }
    }
    elementInjectors[i] = elementInjector;
    if (binder.hasElementPropertyBindings) {
      ListWrapper.push(elementsWithPropertyBindings, element);
    }
    var textNodeIndices = binder.textNodeIndices;
    if (isPresent(textNodeIndices)) {
      var childNode = DOM.firstChild(DOM.templateAwareRoot(element));
      for (var j = 0,
          k = 0; j < textNodeIndices.length; j++) {
        for (var index = textNodeIndices[j]; k < index; k++) {
          childNode = DOM.nextSibling(childNode);
        }
        ListWrapper.push(textNodes, childNode);
      }
    }
    var lightDom = null;
    var bindingPropagationConfig = null;
    if (isPresent(binder.componentDirective)) {
      var strategy = this.shadowDomStrategy;
      var childView = binder.nestedProtoView.instantiate(elementInjector);
      view.changeDetector.addChild(childView.changeDetector);
      lightDom = strategy.constructLightDom(view, childView, element);
      strategy.attachTemplate(element, childView);
      bindingPropagationConfig = new BindingPropagationConfig(view.changeDetector);
      ListWrapper.push(componentChildViews, childView);
    }
    var viewPort = null;
    if (isPresent(binder.templateDirective)) {
      var destLightDom = this._parentElementLightDom(protoElementInjector, preBuiltObjects);
      viewPort = new ViewPort(view, element, binder.nestedProtoView, elementInjector, destLightDom);
      ListWrapper.push(viewPorts, viewPort);
    }
    if (isPresent(elementInjector)) {
      preBuiltObjects[i] = new PreBuiltObjects(view, new NgElement(element), viewPort, lightDom, bindingPropagationConfig);
    }
    if (isPresent(binder.events)) {
      MapWrapper.forEach(binder.events, (expr, eventName) {
        if (isBlank(elementInjector) || !elementInjector.hasEventEmitter(eventName)) {
          ProtoView._addNativeEventListener(element, eventName, expr, view);
        }
      });
    }
  }
  view.init(elementInjectors, rootElementInjectors, textNodes, elementsWithPropertyBindings, viewPorts, preBuiltObjects, componentChildViews);
  return view;
}
static _addNativeEventListener(Element element, String eventName, AST expr, View view) {
  var locals = MapWrapper.create();
  var innerCallback = ProtoView.buildInnerCallback(expr, view, locals);
  DOM.on(element, eventName, (event) {
    if (identical(event.target, element)) {
      innerCallback(event);
    }
  });
}
static buildInnerCallback(AST expr, View view, Map locals) {
  return (event) {
    if (view.hydrated()) {
      MapWrapper.set(locals, '''\$event''', event);
      var context = new ContextWithVariableBindings(view.context, locals);
      expr.eval(context);
    }
  };
}
LightDom _parentElementLightDom(ProtoElementInjector protoElementInjector, List preBuiltObjects) {
  var p = protoElementInjector.parent;
  return isPresent(p) ? preBuiltObjects[p.index].lightDom : null;
}
bindVariable(String contextName, String templateName) {
  MapWrapper.set(this.variableBindings, contextName, templateName);
  MapWrapper.set(this.protoContextLocals, templateName, null);
}
ElementBinder bindElement(ProtoElementInjector protoElementInjector, [DirectiveMetadata componentDirective = null, DirectiveMetadata templateDirective = null]) {
  var elBinder = new ElementBinder(protoElementInjector, componentDirective, templateDirective);
  ListWrapper.push(this.elementBinders, elBinder);
  return elBinder;
}
bindTextNode(int indexInParent, AST expression) {
  var elBinder = this.elementBinders[this.elementBinders.length - 1];
  if (isBlank(elBinder.textNodeIndices)) {
    elBinder.textNodeIndices = ListWrapper.create();
  }
  ListWrapper.push(elBinder.textNodeIndices, indexInParent);
  var memento = this.textNodesWithBindingCount++;
  this.protoChangeDetector.addAst(expression, memento, memento);
}
bindElementProperty(AST expression, String setterName, SetterFn setter) {
  var elBinder = this.elementBinders[this.elementBinders.length - 1];
  if (!elBinder.hasElementPropertyBindings) {
    elBinder.hasElementPropertyBindings = true;
    this.elementsWithBindingCount++;
  }
  var memento = new ElementPropertyMemento(this.elementsWithBindingCount - 1, setterName, setter);
  this.protoChangeDetector.addAst(expression, memento, memento);
}
bindEvent(String eventName, AST expression) {
  var elBinder = this.elementBinders[this.elementBinders.length - 1];
  if (isBlank(elBinder.events)) {
    elBinder.events = MapWrapper.create();
  }
  MapWrapper.set(elBinder.events, eventName, expression);
}
bindDirectiveProperty(num directiveIndex, AST expression, String setterName, SetterFn setter, bool isContentWatch) {
  var expMemento = new DirectivePropertyMemento(this.elementBinders.length - 1, directiveIndex, setterName, setter);
  var groupMemento = DirectivePropertyGroupMemento.get(expMemento);
  this.protoChangeDetector.addAst(expression, expMemento, groupMemento, isContentWatch);
}
static ProtoView createRootProtoView(ProtoView protoView, insertionElement, DirectiveMetadata rootComponentAnnotatedType, ProtoChangeDetector protoChangeDetector, ShadowDomStrategy shadowDomStrategy) {
  DOM.addClass(insertionElement, 'ng-binding');
  var rootProtoView = new ProtoView(insertionElement, protoChangeDetector, shadowDomStrategy);
  rootProtoView.instantiateInPlace = true;
  var binder = rootProtoView.bindElement(new ProtoElementInjector(null, 0, [rootComponentAnnotatedType.type], true));
  binder.componentDirective = rootComponentAnnotatedType;
  binder.nestedProtoView = protoView;
  return rootProtoView;
}
}
class ElementPropertyMemento {
int _elementIndex;
String _setterName;
SetterFn _setter;
ElementPropertyMemento(int elementIndex, String setterName, SetterFn setter) {
  this._elementIndex = elementIndex;
  this._setterName = setterName;
  this._setter = setter;
}
invoke(ChangeRecord record, List bindElements) {
  Element element = bindElements[this._elementIndex];
  this._setter(element, record.currentValue);
}
}
class DirectivePropertyMemento {
int _elementInjectorIndex;
int _directiveIndex;
String _setterName;
SetterFn _setter;
DirectivePropertyMemento(num elementInjectorIndex, num directiveIndex, String setterName, SetterFn setter) {
  this._elementInjectorIndex = elementInjectorIndex;
  this._directiveIndex = directiveIndex;
  this._setterName = setterName;
  this._setter = setter;
}
invoke(ChangeRecord record, List elementInjectors) {
  ElementInjector elementInjector = elementInjectors[this._elementInjectorIndex];
  var directive = elementInjector.getAtIndex(this._directiveIndex);
  this._setter(directive, record.currentValue);
}
}
var _groups = MapWrapper.create();
class DirectivePropertyGroupMemento {
num _elementInjectorIndex;
num _directiveIndex;
DirectivePropertyGroupMemento(num elementInjectorIndex, num directiveIndex) {
  this._elementInjectorIndex = elementInjectorIndex;
  this._directiveIndex = directiveIndex;
}
static get(DirectivePropertyMemento memento) {
  var elementInjectorIndex = memento._elementInjectorIndex;
  var directiveIndex = memento._directiveIndex;
  var id = elementInjectorIndex * 100 + directiveIndex;
  if (!MapWrapper.contains(_groups, id)) {
    MapWrapper.set(_groups, id, new DirectivePropertyGroupMemento(elementInjectorIndex, directiveIndex));
  }
  return MapWrapper.get(_groups, id);
}
directive(List elementInjectors) {
  ElementInjector elementInjector = elementInjectors[this._elementInjectorIndex];
  return elementInjector.getAtIndex(this._directiveIndex);
}
}
class PropertyUpdate {
var currentValue;
var previousValue;
PropertyUpdate(currentValue, previousValue) {
  this.currentValue = currentValue;
  this.previousValue = previousValue;
}
}
