library angular2.src.di.exceptions;
import 'package:angular2/src/facade/collection.dart' show ListWrapper, List;
import 'package:angular2/src/facade/lang.dart' show stringify;
import './key.dart' show Key;
findFirstClosedCycle(List keys) {
  var res = [];
  for (var i = 0; i < keys.length; ++i) {
    if (ListWrapper.contains(res, keys[i])) {
      ListWrapper.push(res, keys[i]);
      return res;
    } else {
      ListWrapper.push(res, keys[i]);
    }
  }
  return res;
}
constructResolvingPath(List keys) {
  if (keys.length > 1) {
    var reversed = findFirstClosedCycle(ListWrapper.reversed(keys));
    var tokenStrs = ListWrapper.map(reversed, (k) {
      return stringify(k.token);
    });
    return " (" + tokenStrs.join(' -> ') + ")";
  } else {
    return "";
  }
}
class KeyMetadataError extends Error {}
class ProviderError extends Error {
List keys;
Function constructResolvingMessage;
var message;
ProviderError(Key key, Function constructResolvingMessage) {
  this.keys = [key];
  this.constructResolvingMessage = constructResolvingMessage;
  this.message = this.constructResolvingMessage(this.keys);
}
addKey(Key key) {
  ListWrapper.push(this.keys, key);
  this.message = this.constructResolvingMessage(this.keys);
}
toString() {
  return this.message;
}
}
class NoProviderError extends ProviderError {
NoProviderError(Key key): super(key, (List keys) {
  var first = stringify(ListWrapper.first(keys).token);
  return '''No provider for ${first}!${constructResolvingPath(keys)}''';
}) {}
}
class AsyncBindingError extends ProviderError {
AsyncBindingError(Key key): super(key, (List keys) {
  var first = stringify(ListWrapper.first(keys).token);
  return '''Cannot instantiate ${first} synchronously. ''' + '''It is provided as a promise!${constructResolvingPath(keys)}''';
}) {}
}
class CyclicDependencyError extends ProviderError {
CyclicDependencyError(Key key): super(key, (List keys) {
  return '''Cannot instantiate cyclic dependency!${constructResolvingPath(keys)}''';
}) {}
}
class InstantiationError extends ProviderError {
InstantiationError(originalException, Key key): super(key, (List keys) {
  var first = stringify(ListWrapper.first(keys).token);
  return '''Error during instantiation of ${first}!${constructResolvingPath(keys)}.''' + ''' ORIGINAL ERROR: ${originalException}''';
}) {}
}
class InvalidBindingError extends Error {
String message;
InvalidBindingError(binding) {
  this.message = '''Invalid binding ${binding}''';
}
toString() {
  return this.message;
}
}
class NoAnnotationError extends Error {
String message;
NoAnnotationError(typeOrFunc) {
  this.message = '''Cannot resolve all parameters for ${stringify(typeOrFunc)}''';
}
toString() {
  return this.message;
}
}
