library angular2.src.directives.ng_repeat;
import 'package:angular2/src/core/annotations/annotations.dart' show Template;
import 'package:angular2/src/core/compiler/interfaces.dart' show OnChange;
import 'package:angular2/src/core/compiler/viewport.dart' show ViewPort;
import 'package:angular2/src/core/compiler/view.dart' show View;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank;
import 'package:angular2/src/facade/collection.dart' show ListWrapper;
@Template(selector: '[ng-repeat]', bind: const{'in': 'iterable[]'}) 
class NgRepeat extends OnChange {
ViewPort viewPort;
var iterable;
NgRepeat(ViewPort viewPort) {
  this.viewPort = viewPort;
}
onChange(changes) {
  var iteratorChanges = changes['iterable'];
  if (isBlank(iteratorChanges) || isBlank(iteratorChanges.currentValue)) {
    this.viewPort.clear();
    return ;
  }
  var recordViewTuples = [];
  iteratorChanges.currentValue.forEachRemovedItem((removedRecord) {
    return ListWrapper.push(recordViewTuples, new RecordViewTuple(removedRecord, null));
  });
  iteratorChanges.currentValue.forEachMovedItem((movedRecord) {
    return ListWrapper.push(recordViewTuples, new RecordViewTuple(movedRecord, null));
  });
  var insertTuples = NgRepeat.bulkRemove(recordViewTuples, this.viewPort);
  iteratorChanges.currentValue.forEachAddedItem((addedRecord) {
    return ListWrapper.push(insertTuples, new RecordViewTuple(addedRecord, null));
  });
  NgRepeat.bulkInsert(insertTuples, this.viewPort);
  for (var i = 0; i < insertTuples.length; i++) {
    this.perViewChange(insertTuples[i].view, insertTuples[i].record);
  }
}
perViewChange(view, record) {
  view.setLocal('\$implicit', record.item);
  view.setLocal('index', record.currentIndex);
}
static bulkRemove(tuples, viewPort) {
  tuples.sort((a, b) {
    return a.record.previousIndex - b.record.previousIndex;
  });
  var movedTuples = [];
  for (var i = tuples.length - 1; i >= 0; i--) {
    var tuple = tuples[i];
    if (isPresent(tuple.record.currentIndex)) {
      tuple.view = viewPort.detach(tuple.record.previousIndex);
      ListWrapper.push(movedTuples, tuple);
    } else {
      viewPort.remove(tuple.record.previousIndex);
    }
  }
  return movedTuples;
}
static bulkInsert(tuples, viewPort) {
  tuples.sort((a, b) {
    return a.record.currentIndex - b.record.currentIndex;
  });
  for (var i = 0; i < tuples.length; i++) {
    var tuple = tuples[i];
    if (isPresent(tuple.view)) {
      viewPort.insert(tuple.view, tuple.record.currentIndex);
    } else {
      tuple.view = viewPort.create(tuple.record.currentIndex);
    }
  }
  return tuples;
}
}
class RecordViewTuple {
View view;
dynamic record;
RecordViewTuple(record, view) {
  this.record = record;
  this.view = view;
}
}
