library angular2.src.directives.ng_switch;
import 'package:angular2/src/core/annotations/annotations.dart' show Decorator, Template;
import 'package:angular2/src/core/compiler/viewport.dart' show ViewPort;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/lang.dart' show isPresent, isBlank;
import 'package:angular2/src/facade/collection.dart' show ListWrapper, List, MapWrapper, Map;
import 'package:angular2/src/core/annotations/visibility.dart' show Parent;
@Decorator(selector: '[ng-switch]', bind: const{'ng-switch': 'value'}) 
class NgSwitch {
dynamic _switchValue;
bool _useDefault;
Map _valueViewPorts;
List _activeViewPorts;
NgSwitch() {
  this._valueViewPorts = MapWrapper.create();
  this._activeViewPorts = ListWrapper.create();
  this._useDefault = false;
}
set value(value) {
  this._removeAllActiveViewPorts();
  this._useDefault = false;
  var viewPorts = MapWrapper.get(this._valueViewPorts, value);
  if (isBlank(viewPorts)) {
    this._useDefault = true;
    viewPorts = MapWrapper.get(this._valueViewPorts, _whenDefault);
  }
  this._activateViewPorts(viewPorts);
  this._switchValue = value;
}
_onWhenValueChanged(oldWhen, newWhen, ViewPort viewPort) {
  this._deregisterViewPort(oldWhen, viewPort);
  this._registerViewPort(newWhen, viewPort);
  if (identical(oldWhen, this._switchValue)) {
    viewPort.remove();
    ListWrapper.remove(this._activeViewPorts, viewPort);
  } else if (identical(newWhen, this._switchValue)) {
    if (this._useDefault) {
      this._useDefault = false;
      this._removeAllActiveViewPorts();
    }
    viewPort.create();
    ListWrapper.push(this._activeViewPorts, viewPort);
  }
  if (identical(this._activeViewPorts.length, 0) && !this._useDefault) {
    this._useDefault = true;
    this._activateViewPorts(MapWrapper.get(this._valueViewPorts, _whenDefault));
  }
}
_removeAllActiveViewPorts() {
  var activeViewPorts = this._activeViewPorts;
  for (var i = 0; i < activeViewPorts.length; i++) {
    activeViewPorts[i].remove();
  }
  this._activeViewPorts = ListWrapper.create();
}
_activateViewPorts(viewPorts) {
  if (isPresent(viewPorts)) {
    for (var i = 0; i < viewPorts.length; i++) {
      viewPorts[i].create();
    }
    this._activeViewPorts = viewPorts;
  }
}
_registerViewPort(value, ViewPort viewPort) {
  var viewPorts = MapWrapper.get(this._valueViewPorts, value);
  if (isBlank(viewPorts)) {
    viewPorts = ListWrapper.create();
    MapWrapper.set(this._valueViewPorts, value, viewPorts);
  }
  ListWrapper.push(viewPorts, viewPort);
}
_deregisterViewPort(value, ViewPort viewPort) {
  if (value == _whenDefault)
    return ;
  var viewPorts = MapWrapper.get(this._valueViewPorts, value);
  if (viewPorts.length == 1) {
    MapWrapper.delete(this._valueViewPorts, value);
  } else {
    ListWrapper.remove(viewPorts, viewPort);
  }
}
}
@Template(selector: '[ng-switch-when]', bind: const{'ng-switch-when': 'when'}) 
class NgSwitchWhen {
dynamic _value;
NgSwitch _ngSwitch;
ViewPort _viewPort;
NgSwitchWhen(NgElement el, ViewPort viewPort, @Parent() NgSwitch ngSwitch) {
  this._value = _whenDefault;
  this._ngSwitch = ngSwitch;
  this._viewPort = viewPort;
}
set when(value) {
  this._ngSwitch._onWhenValueChanged(this._value, value, this._viewPort);
  this._value = value;
}
}
@Template(selector: '[ng-switch-default]') 
class NgSwitchDefault {
NgSwitchDefault(ViewPort viewPort, @Parent() NgSwitch ngSwitch) {
  ngSwitch._registerViewPort(_whenDefault, viewPort);
}
}
var _whenDefault = new Object();
