library angular2.test.core.compiler.directive_metadata_reader_spec;
import 'package:angular2/test_lib.dart' show ddescribe, describe, it, iit, expect, beforeEach;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/core/annotations/annotations.dart' show Decorator, Component;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/compiler/directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show ShadowDomStrategy, NativeShadowDomStrategy;
import 'package:angular2/src/facade/lang.dart' show CONST;
@Decorator(selector: 'someSelector') 
class SomeDirective {}
class SomeDirectiveWithoutAnnotation {}
@Component(selector: 'withoutDirectives') 
class ComponentWithoutDirectives {}
@Component(selector: 'withDirectives', template: const TemplateConfig(directives: const[ComponentWithoutDirectives])) 
class ComponentWithDirectives {}
main() {
  describe("DirectiveMetadataReader", () {
    var reader;
    beforeEach(() {
      reader = new DirectiveMetadataReader();
    });
    it('should read out the annotation', () {
      var directiveMetadata = reader.read(SomeDirective);
      expect(directiveMetadata).toEqual(new DirectiveMetadata(SomeDirective, new Decorator(selector: 'someSelector'), null));
    });
    it('should throw if not matching annotation is found', () {
      expect(() {
        reader.read(SomeDirectiveWithoutAnnotation);
      }).toThrowError('No Directive annotation found on SomeDirectiveWithoutAnnotation');
    });
    describe("componentDirectives", () {
      it("should return an empty list when no directives specified", () {
        var cmp = reader.read(ComponentWithoutDirectives);
        expect(cmp.componentDirectives).toEqual([]);
      });
      it("should return a list of directives specified in the template config", () {
        var cmp = reader.read(ComponentWithDirectives);
        expect(cmp.componentDirectives).toEqual([ComponentWithoutDirectives]);
      });
    });
  });
}
