library angular2.test.core.compiler.integration_spec;
import 'package:angular2/test_lib.dart' show describe, xit, it, expect, beforeEach, ddescribe, iit, el;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart' show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart' show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show NativeShadowDomStrategy;
import 'package:angular2/src/core/compiler/template_loader.dart' show TemplateLoader;
import 'package:angular2/src/core/annotations/annotations.dart' show Decorator, Component, Template;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/compiler/viewport.dart' show ViewPort;
import 'package:angular2/src/facade/collection.dart' show MapWrapper;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;
main() {
  describe('integration tests', () {
    var compiler;
    beforeEach(() {
      compiler = new Compiler(dynamicChangeDetection, new TemplateLoader(new XHRMock()), new DirectiveMetadataReader(), new Parser(new Lexer()), new CompilerCache(), new NativeShadowDomStrategy());
    });
    describe('react to record changes', () {
      var view;
      var ctx;
      var cd;
      createView(pv) {
        ctx = new MyComp();
        view = pv.instantiate(null);
        view.hydrate(new Injector([]), null, ctx);
        cd = view.changeDetector;
      }
      it('should consume text node changes', (done) {
        compiler.compile(MyComp, el('<div>{{ctxProp}}</div>')).then((pv) {
          createView(pv);
          ctx.ctxProp = 'Hello World!';
          cd.detectChanges();
          expect(DOM.getInnerHTML(view.nodes[0])).toEqual('Hello World!');
          done();
        });
      });
      it('should consume element binding changes', (done) {
        compiler.compile(MyComp, el('<div [id]="ctxProp"></div>')).then((pv) {
          createView(pv);
          ctx.ctxProp = 'Hello World!';
          cd.detectChanges();
          expect(view.nodes[0].id).toEqual('Hello World!');
          done();
        });
      });
      it('should consume directive watch expression change.', (done) {
        compiler.compile(MyComp, el('<div my-dir [elprop]="ctxProp"></div>')).then((pv) {
          createView(pv);
          ctx.ctxProp = 'Hello World!';
          cd.detectChanges();
          var elInj = view.elementInjectors[0];
          expect(elInj.get(MyDir).dirProp).toEqual('Hello World!');
          done();
        });
      });
      it('should support nested components.', (done) {
        compiler.compile(MyComp, el('<child-cmp></child-cmp>')).then((pv) {
          createView(pv);
          cd.detectChanges();
          expect(view.nodes[0].shadowRoot.childNodes[0].nodeValue).toEqual('hello');
          done();
        });
      });
      it('should support different directive types on a single node', (done) {
        compiler.compile(MyComp, el('<child-cmp my-dir [elprop]="ctxProp"></child-cmp>')).then((pv) {
          createView(pv);
          ctx.ctxProp = 'Hello World!';
          cd.detectChanges();
          var elInj = view.elementInjectors[0];
          expect(elInj.get(MyDir).dirProp).toEqual('Hello World!');
          expect(elInj.get(ChildComp).dirProp).toEqual(null);
          done();
        });
      });
      it('should support template directives via `<template>` elements.', (done) {
        compiler.compile(MyComp, el('<div><template some-tmplate var-greeting="some-tmpl"><copy-me>{{greeting}}</copy-me></template></div>')).then((pv) {
          createView(pv);
          cd.detectChanges();
          var childNodesOfWrapper = view.nodes[0].childNodes;
          expect(childNodesOfWrapper.length).toBe(3);
          expect(childNodesOfWrapper[1].childNodes[0].nodeValue).toEqual('hello');
          expect(childNodesOfWrapper[2].childNodes[0].nodeValue).toEqual('again');
          done();
        });
      });
      it('should support template directives via `template` attribute.', (done) {
        compiler.compile(MyComp, el('<div><copy-me template="some-tmplate: var greeting=some-tmpl">{{greeting}}</copy-me></div>')).then((pv) {
          createView(pv);
          cd.detectChanges();
          var childNodesOfWrapper = view.nodes[0].childNodes;
          expect(childNodesOfWrapper.length).toBe(3);
          expect(childNodesOfWrapper[1].childNodes[0].nodeValue).toEqual('hello');
          expect(childNodesOfWrapper[2].childNodes[0].nodeValue).toEqual('again');
          done();
        });
      });
    });
  });
}
@Decorator(selector: '[my-dir]', bind: const{'elprop': 'dirProp'}) 
class MyDir {
String dirProp;
MyDir() {
  this.dirProp = '';
}
}
@Component(template: const TemplateConfig(directives: const[MyDir, ChildComp, SomeTemplate])) 
class MyComp {
String ctxProp;
MyComp() {
  this.ctxProp = 'initial value';
}
}
@Component(selector: 'child-cmp', componentServices: const[MyService], template: const TemplateConfig(directives: const[MyDir], inline: '{{ctxProp}}')) 
class ChildComp {
String ctxProp;
String dirProp;
ChildComp(MyService service) {
  this.ctxProp = service.greeting;
  this.dirProp = null;
}
}
@Template(selector: '[some-tmplate]') 
class SomeTemplate {
SomeTemplate(ViewPort viewPort) {
  viewPort.create().setLocal('some-tmpl', 'hello');
  viewPort.create().setLocal('some-tmpl', 'again');
}
}
class MyService {
String greeting;
MyService() {
  this.greeting = 'hello';
}
}
