library angular2.test.core.compiler.pipeline.property_binding_parser_spec;
import 'package:angular2/test_lib.dart' show describe, beforeEach, it, expect, iit, ddescribe, el;
import 'package:angular2/src/core/compiler/pipeline/property_binding_parser.dart' show PropertyBindingParser;
import 'package:angular2/src/core/compiler/pipeline/compile_pipeline.dart' show CompilePipeline;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/src/facade/collection.dart' show MapWrapper;
import 'package:angular2/change_detection.dart' show Lexer, Parser;
main() {
  describe('PropertyBindingParser', () {
    createPipeline() {
      return new CompilePipeline([new PropertyBindingParser(new Parser(new Lexer()), null)]);
    }
    it('should detect [] syntax', () {
      var results = createPipeline().process(el('<div [a]="b"></div>'));
      expect(MapWrapper.get(results[0].propertyBindings, 'a').source).toEqual('b');
    });
    it('should detect bind- syntax', () {
      var results = createPipeline().process(el('<div bind-a="b"></div>'));
      expect(MapWrapper.get(results[0].propertyBindings, 'a').source).toEqual('b');
    });
    it('should detect interpolation syntax', () {
      var results = createPipeline().process(el('<div a="{{b}}"></div>'));
      expect(MapWrapper.get(results[0].propertyBindings, 'a').source).toEqual('{{b}}');
    });
    it('should detect var- syntax', () {
      var results = createPipeline().process(el('<template var-a="b"></template>'));
      expect(MapWrapper.get(results[0].variableBindings, 'b')).toEqual('a');
    });
    it('should not allow var- syntax on non template elements', () {
      expect(() {
        createPipeline().process(el('<div var-a="b"></div>'));
      }).toThrowError('var-* is only allowed on <template> elements!');
    });
    it('should detect () syntax', () {
      var results = createPipeline().process(el('<div (click)="b()"></div>'));
      expect(MapWrapper.get(results[0].eventBindings, 'click').source).toEqual('b()');
      results = createPipeline().process(el('<div (click[])="b()"></div>'));
      expect(MapWrapper.get(results[0].eventBindings, 'click[]').source).toEqual('b()');
    });
    it('should detect on- syntax', () {
      var results = createPipeline().process(el('<div on-click="b()"></div>'));
      expect(MapWrapper.get(results[0].eventBindings, 'click').source).toEqual('b()');
    });
  });
}
