library angular2.test.core.compiler.shadow_dom.shadow_dom_emulation_integration_spec;
import 'package:angular2/test_lib.dart' show describe, xit, it, expect, beforeEach, ddescribe, iit, el;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart' show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart' show Compiler, CompilerCache;
import 'package:angular2/src/core/life_cycle/life_cycle.dart' show LifeCycle;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show ShadowDomStrategy, NativeShadowDomStrategy, EmulatedShadowDomStrategy;
import 'package:angular2/src/core/compiler/template_loader.dart' show TemplateLoader;
import 'package:angular2/src/core/annotations/annotations.dart' show Decorator, Component, Template;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/compiler/viewport.dart' show ViewPort;
import 'package:angular2/src/facade/collection.dart' show StringMapWrapper, MapWrapper;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;
main() {
  describe('integration tests', () {
    StringMapWrapper.forEach({
      "native": new NativeShadowDomStrategy(),
      "emulated": new EmulatedShadowDomStrategy()
    }, (strategy, name) {
      describe('''${name} shadow dom strategy''', () {
        var compiler;
        beforeEach(() {
          compiler = new Compiler(dynamicChangeDetection, new TemplateLoader(new XHRMock()), new DirectiveMetadataReader(), new Parser(new Lexer()), new CompilerCache(), strategy);
        });
        compile(template, assertions) {
          compiler.compile(MyComp, el(template)).then(createView).then((view) {
            var lc = new LifeCycle(view.changeDetector, false);
            assertions(view, lc);
          });
        }
        it('should support multiple content tags', (done) {
          var temp = '<multiple-content-tags>' + '<div>B</div>' + '<div>C</div>' + '<div class="left">A</div>' + '</multiple-content-tags>';
          compile(temp, (view, lc) {
            expect(view.nodes).toHaveText('(A, BC)');
            done();
          });
        });
        it('should redistribute only direct children', (done) {
          var temp = '<multiple-content-tags>' + '<div>B<div class="left">A</div></div>' + '<div>C</div>' + '</multiple-content-tags>';
          compile(temp, (view, lc) {
            expect(view.nodes).toHaveText('(, BAC)');
            done();
          });
        });
        it("should redistribute when the light dom changes", (done) {
          var temp = '<multiple-content-tags>' + '<div template="manual" class="left">A</div>' + '<div>B</div>' + '</multiple-content-tags>';
          compile(temp, (view, lc) {
            var dir = view.elementInjectors[1].get(ManualTemplateDirective);
            expect(view.nodes).toHaveText('(, B)');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('(A, B)');
            dir.hide();
            lc.tick();
            expect(view.nodes).toHaveText('(, B)');
            done();
          });
        });
        it("should support nested components", (done) {
          var temp = '<outer-with-indirect-nested>' + '<div>A</div>' + '<div>B</div>' + '</outer-with-indirect-nested>';
          compile(temp, (view, lc) {
            expect(view.nodes).toHaveText('OUTER(SIMPLE(AB))');
            done();
          });
        });
        it("should support nesting with content being direct child of a nested component", (done) {
          var temp = '<outer>' + '<div template="manual" class="left">A</div>' + '<div>B</div>' + '<div>C</div>' + '</outer>';
          compile(temp, (view, lc) {
            var dir = view.elementInjectors[1].get(ManualTemplateDirective);
            expect(view.nodes).toHaveText('OUTER(INNER(INNERINNER(,BC)))');
            dir.show();
            lc.tick();
            expect(view.nodes).toHaveText('OUTER(INNER(INNERINNER(A,BC)))');
            done();
          });
        });
      });
    });
  });
}
class TestDirectiveMetadataReader extends DirectiveMetadataReader {
var shadowDomStrategy;
TestDirectiveMetadataReader(shadowDomStrategy) {
  this.shadowDomStrategy = shadowDomStrategy;
}
ShadowDomStrategy parseShadowDomStrategy(Component annotation) {
  return this.shadowDomStrategy;
}
}
@Template(selector: '[manual]') 
class ManualTemplateDirective {
var viewPort;
ManualTemplateDirective(ViewPort viewPort) {
  this.viewPort = viewPort;
}
show() {
  this.viewPort.create();
}
hide() {
  this.viewPort.remove(0);
}
}
@Template(selector: '[auto]', bind: const{'auto': 'auto'}) 
class AutoTemplateDirective {
var viewPort;
AutoTemplateDirective(ViewPort viewPort) {
  this.viewPort = viewPort;
}
set auto(bool newValue) {
  if (newValue) {
    this.viewPort.create();
  } else {
    this.viewPort.remove(0);
  }
}
}
@Component(selector: 'simple', template: const TemplateConfig(inline: 'SIMPLE(<content></content>)')) 
class Simple {}
@Component(selector: 'multiple-content-tags', template: const TemplateConfig(inline: '(<content select=".left"></content>, <content></content>)')) 
class MultipleContentTagsComponent {}
@Component(selector: 'conditional-content', template: const TemplateConfig(inline: '<div>(<div template="auto: cond"><content select=".left"></content></div>, <content></content>)</div>', directives: const[AutoTemplateDirective])) 
class ConditionalContentComponent {
bool cond;
ConditionalContentComponent() {
  this.cond = false;
}
showLeft() {
  this.cond = true;
}
hideLeft() {
  this.cond = false;
}
}
@Component(selector: 'outer-with-indirect-nested', template: const TemplateConfig(inline: 'OUTER(<simple><div><content></content></div></simple>)', directives: const[Simple])) 
class OuterWithIndirectNestedComponent {}
@Component(selector: 'outer', template: const TemplateConfig(inline: 'OUTER(<inner><content></content></inner>)', directives: const[InnerComponent])) 
class OuterComponent {}
@Component(selector: 'inner', template: const TemplateConfig(inline: 'INNER(<innerinner><content></content></innerinner>)', directives: const[InnerInnerComponent])) 
class InnerComponent {}
@Component(selector: 'innerinner', template: const TemplateConfig(inline: 'INNERINNER(<content select=".left"></content>,<content></content>)')) 
class InnerInnerComponent {}
@Component(selector: 'my-comp', template: const TemplateConfig(directives: const[MultipleContentTagsComponent, ManualTemplateDirective, ConditionalContentComponent, OuterWithIndirectNestedComponent, OuterComponent])) 
class MyComp {}
createView(pv) {
  var view = pv.instantiate(null);
  view.hydrate(new Injector([]), null, {});
  return view;
}
