library angular2.test.core.compiler.template_loader_spec;
import 'package:angular2/test_lib.dart' show describe, it, expect, beforeEach, ddescribe, iit, xit, el;
import 'package:angular2/src/core/compiler/template_loader.dart' show TemplateLoader;
import 'package:angular2/src/core/annotations/annotations.dart' show Component;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/compiler/directive_metadata.dart' show DirectiveMetadata;
import 'package:angular2/src/facade/async.dart' show PromiseWrapper;
import 'package:angular2/src/mock/xhr_mock.dart' show XHRMock;
main() {
  describe('TemplateLoader', () {
    var loader;
    var xhr;
    beforeEach(() {
      xhr = new XHRMock();
      loader = new TemplateLoader(xhr);
    });
    createMetadata({inline : null,
      url : null}) {
      var config = new TemplateConfig(url: url, inline: inline);
      var component = new Component(template: config);
      return new DirectiveMetadata(FakeComponent, component, null);
    }
    it('should load inline templates', (done) {
      var template = 'inline template';
      var md = createMetadata(inline: template);
      loader.load(md).then((el) {
        expect(el.content).toHaveText(template);
        done();
      });
    });
    it('should load templates through XHR', (done) {
      var url = '/foo';
      var template = 'xhr template';
      xhr.expect(url, template);
      var md = createMetadata(url: '/foo');
      loader.load(md).then((el) {
        expect(el.content).toHaveText(template);
        done();
      });
      xhr.flush();
    });
    it('should cache template loaded through XHR', (done) {
      var firstEl;
      var url = '/foo';
      var template = 'xhr template';
      xhr.expect(url, template);
      var md = createMetadata(url: '/foo');
      loader.load(md).then((el) {
        firstEl = el;
        return loader.load(md);
      }).then((el) {
        expect(el).toBe(firstEl);
        expect(el.content).toHaveText(template);
        done();
      });
      xhr.flush();
    });
    it('should throw when no template is defined', () {
      var md = createMetadata();
      expect(() {
        return loader.load(md);
      }).toThrowError('No template configured for component FakeComponent');
    });
    it('should return a rejected Promise when xhr loading fails', (done) {
      var url = '/foo';
      xhr.expect(url, null);
      var md = createMetadata(url: '/foo');
      PromiseWrapper.then(loader.load(md), (_) {
        throw 'Unexpected response';
      }, (error) {
        expect(error).toEqual('Failed to load /foo');
        done();
      });
      xhr.flush();
    });
  });
}
class FakeComponent {}
