library angular2.test.directives.ng_non_bindable_spec;
import 'package:angular2/test_lib.dart' show describe, xit, it, expect, beforeEach, ddescribe, iit, el;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/change_detection.dart' show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart' show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show NativeShadowDomStrategy;
import 'package:angular2/src/core/annotations/annotations.dart' show Decorator, Component;
import 'package:angular2/src/core/annotations/template_config.dart' show TemplateConfig;
import 'package:angular2/src/core/dom/element.dart' show NgElement;
import 'package:angular2/src/directives/ng_non_bindable.dart' show NgNonBindable;
main() {
  describe('ng-non-bindable', () {
    var view;
    var cd;
    var compiler;
    var component;
    beforeEach(() {
      compiler = new Compiler(dynamicChangeDetection, null, new DirectiveMetadataReader(), new Parser(new Lexer()), new CompilerCache(), new NativeShadowDomStrategy());
    });
    createView(pv) {
      component = new TestComponent();
      view = pv.instantiate(null);
      view.hydrate(new Injector([]), null, component);
      cd = view.changeDetector;
    }
    compileWithTemplate(template) {
      return compiler.compile(TestComponent, el(template));
    }
    it('should not interpolate children', (done) {
      var template = '<div>{{text}}<span ng-non-bindable>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        expect(DOM.getText(view.nodes[0])).toEqual('foo{{text}}');
        done();
      });
    });
    it('should ignore directives on child nodes', (done) {
      var template = '<div ng-non-bindable><span id=child test-dec>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        var span = DOM.querySelector(view.nodes[0], '#child');
        expect(DOM.hasClass(span, 'compiled')).toBeFalsy();
        done();
      });
    });
    it('should trigger directives on the same node', (done) {
      var template = '<div><span id=child ng-non-bindable test-dec>{{text}}</span></div>';
      compileWithTemplate(template).then((pv) {
        createView(pv);
        cd.detectChanges();
        var span = DOM.querySelector(view.nodes[0], '#child');
        expect(DOM.hasClass(span, 'compiled')).toBeTruthy();
        done();
      });
    });
  });
}
@Component(selector: 'test-cmp', template: const TemplateConfig(inline: '', directives: const[NgNonBindable, TestDecorator])) 
class TestComponent {
String text;
TestComponent() {
  this.text = 'foo';
}
}
@Decorator(selector: '[test-dec]') 
class TestDecorator {
TestDecorator(NgElement el) {
  DOM.addClass(el.domElement, 'compiled');
}
}
