library angular2.test.forms.integration_spec;
import 'package:angular2/test_lib.dart' show ddescribe, describe, it, iit, xit, expect, beforeEach, afterEach, el, queryView, dispatchEvent;
import 'package:angular2/change_detection.dart' show Lexer, Parser, ChangeDetector, dynamicChangeDetection;
import 'package:angular2/src/core/compiler/compiler.dart' show Compiler, CompilerCache;
import 'package:angular2/src/core/compiler/directive_metadata_reader.dart' show DirectiveMetadataReader;
import 'package:angular2/src/core/compiler/shadow_dom_strategy.dart' show NativeShadowDomStrategy;
import 'package:angular2/di.dart' show Injector;
import 'package:angular2/src/facade/dom.dart' show DOM;
import 'package:angular2/core.dart' show Component, TemplateConfig;
import 'package:angular2/forms.dart' show ControlDecorator, ControlGroupDecorator, Control, ControlGroup;
main() {
  detectChanges(view) {
    view.changeDetector.detectChanges();
  }
  compile(componentType, template, context, callback) {
    var compiler = new Compiler(dynamicChangeDetection, null, new DirectiveMetadataReader(), new Parser(new Lexer()), new CompilerCache(), new NativeShadowDomStrategy());
    compiler.compile(componentType, el(template)).then((pv) {
      var view = pv.instantiate(null);
      view.hydrate(new Injector([]), null, context);
      detectChanges(view);
      callback(view);
    });
  }
  describe("integration tests", () {
    it("should initialize DOM elements with the given form object", (done) {
      var ctx = new MyComp(new ControlGroup({"login": new Control("loginValue")}));
      var t = '''<div [control-group]=\"form\">
                <input [control-name]=\"\'login\'\">
              </div>''';
      compile(MyComp, t, ctx, (view) {
        var input = queryView(view, "input");
        expect(input.value).toEqual("loginValue");
        done();
      });
    });
    it("should update the control group values on DOM change", (done) {
      var form = new ControlGroup({"login": new Control("oldValue")});
      var ctx = new MyComp(form);
      var t = '''<div [control-group]=\"form\">
                <input [control-name]=\"\'login\'\">
              </div>''';
      compile(MyComp, t, ctx, (view) {
        var input = queryView(view, "input");
        input.value = "updatedValue";
        dispatchEvent(input, "change");
        expect(form.value).toEqual({"login": "updatedValue"});
        done();
      });
    });
    it("should update DOM elements when rebinding the control group", (done) {
      var form = new ControlGroup({"login": new Control("oldValue")});
      var ctx = new MyComp(form);
      var t = '''<div [control-group]=\"form\">
                <input [control-name]=\"\'login\'\">
              </div>''';
      compile(MyComp, t, ctx, (view) {
        ctx.form = new ControlGroup({"login": new Control("newValue")});
        detectChanges(view);
        var input = queryView(view, "input");
        expect(input.value).toEqual("newValue");
        done();
      });
    });
    it("should update DOM element when rebinding the control name", (done) {
      var ctx = new MyComp(new ControlGroup({
        "one": new Control("one"),
        "two": new Control("two")
      }), "one");
      var t = '''<div [control-group]=\"form\">
                <input [control-name]=\"name\">
              </div>''';
      compile(MyComp, t, ctx, (view) {
        var input = queryView(view, "input");
        expect(input.value).toEqual("one");
        ctx.name = "two";
        detectChanges(view);
        expect(input.value).toEqual("two");
        done();
      });
    });
  });
}
@Component(template: const TemplateConfig(directives: const[ControlGroupDecorator, ControlDecorator])) 
class MyComp {
ControlGroup form;
String name;
MyComp(form, [name = null]) {
  this.form = form;
  this.name = name;
}
}
